/*********************************************
 * BIOE-GUI - a frontend for the BIOE system, WIN32/UNIX
 * Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
 * 
 * License: CC-GNU GPL
 * http://creativecommons.org/licenses/GPL/2.0/ 
 *********************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include "bioe.h"
#include "BIOEDeviceWidget.h"

BIOEDeviceWidget::BIOEDeviceWidget(int x, int y, int adr)
  :Fl_Group(x,y,310,320,"")
{
  char label[30];
  this->adr=adr;
  sprintf(label, "Device%d", adr);

  box(FL_THIN_UP_BOX);
  titlebar_grp=new Fl_Group(this->x()+2,this->y()+2,this->w()-2-3-20,20);
    titlebar_grp->copy_label(label);
    titlebar_grp->align(FL_ALIGN_LEFT +FL_ALIGN_INSIDE);
    titlebar_grp->box(FL_THIN_UP_BOX);
    titlebar_grp->color(FL_DARK_GREEN);
    titlebar_grp->labelsize(11);
    titlebar_grp->labelfont(FL_HELVETICA_BOLD);
    titlebar_grp->labelcolor(FL_WHITE);
  titlebar_grp->end();
  x_btn=new Fl_Button(this->x()+this->w()-2-20,this->y()+2,20,20,"X");
  x_btn->callback(x_btn_scb,this);
  x_btn->labelsize(11);
  x_btn->labelfont(FL_HELVETICA_BOLD);
  level_btn=new Fl_Choice(this->x()+50,this->y()+24,250,25,"Level:");
  level_btn->add("1, no DTN and no Settings");
  level_btn->add("2, DTN but no Settings");
  level_btn->add("3, DTN and Settings");
  level_btn->callback(level_btn_scb,this);
  level_btn->value(0);
  DTN_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*0,40,18,"DTN:");
  DTN_txt->value("0");
  DTN_txt->textsize(11);
  DTN_txt->labelsize(11);

  setdtn_btn=new Fl_Button(this->x()+20,this->y()+55+20*1,70,18,"Set DTN");
  setdtn_btn->callback(setdtn_btn_scb,this);
  setdtn_btn->labelsize(11);

  readsettings_btn=new Fl_Button(this->x()+20,this->y()+55+20*3,70,18,"Read S0..7");
  readsettings_btn->callback(readsettings_btn_scb,this);
  readsettings_btn->labelsize(11);

  s0_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*4,40,18,"S0:");
  s0_txt->value("0");
  s0_txt->textsize(11);
  s0_txt->labelsize(11);

  s1_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*5,40,18,"S1:");
  s1_txt->value("0");
  s1_txt->textsize(11);
  s1_txt->labelsize(11);

  s2_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*6,40,18,"S2:");
  s2_txt->value("0");
  s2_txt->textsize(11);
  s2_txt->labelsize(11);

  s3_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*7,40,18,"S3:");
  s3_txt->value("0");
  s3_txt->textsize(11);
  s3_txt->labelsize(11);

  s4_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*8,40,18,"S4:");
  s4_txt->value("0");
  s4_txt->textsize(11);
  s4_txt->labelsize(11);

  s5_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*9,40,18,"S5:");
  s5_txt->value("0");
  s5_txt->textsize(11);
  s5_txt->labelsize(11);

  s6_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*10,40,18,"S6:");
  s6_txt->value("0");
  s6_txt->textsize(11);
  s6_txt->labelsize(11);

  s7_txt=new Fl_Int_Input(this->x()+50,this->y()+55+20*11,40,18,"S7:");
  s7_txt->value("0");
  s7_txt->textsize(11);
  s7_txt->labelsize(11);

  writesettings_btn=new Fl_Button(this->x()+20,this->y()+55+20*12,70,18,"Write S0..7");
  writesettings_btn->callback(writesettings_btn_scb,this);
  writesettings_btn->labelsize(11);


  ep2in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*0,40,18,"IN-EP2");
  ep2in_txt->value("0");
  ep2in_txt->textsize(11);
  ep2in_txt->labelsize(11);
  ep2in_txt->readonly(1);

  ep2out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*0,40,18,"OUT-EP2");
  ep2out_txt->value("0");
  ep2out_txt->textsize(11);
  ep2out_txt->labelsize(11);
  ep2out_txt->align(FL_ALIGN_RIGHT);

  ep3in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*1,40,18,"IN-EP3");
  ep3in_txt->value("0");
  ep3in_txt->textsize(11);
  ep3in_txt->labelsize(11);
  ep3in_txt->readonly(1);

  ep3out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*1,40,18,"OUT-EP3");
  ep3out_txt->value("0");
  ep3out_txt->textsize(11);
  ep3out_txt->labelsize(11);
  ep3out_txt->align(FL_ALIGN_RIGHT);

  ep4in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*2,40,18,"IN-EP4");
  ep4in_txt->value("0");
  ep4in_txt->textsize(11);
  ep4in_txt->labelsize(11);
  ep4in_txt->readonly(1);

  ep4out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*2,40,18,"OUT-EP4");
  ep4out_txt->value("0");
  ep4out_txt->textsize(11);
  ep4out_txt->labelsize(11);
  ep4out_txt->align(FL_ALIGN_RIGHT);

  ep5in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*3,40,18,"IN-EP5");
  ep5in_txt->value("0");
  ep5in_txt->textsize(11);
  ep5in_txt->labelsize(11);
  ep5in_txt->readonly(1);

  ep5out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*3,40,18,"OUT-EP5");
  ep5out_txt->value("0");
  ep5out_txt->textsize(11);
  ep5out_txt->labelsize(11);
  ep5out_txt->align(FL_ALIGN_RIGHT);

  ep6in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*4,40,18,"IN-EP6");
  ep6in_txt->value("0");
  ep6in_txt->textsize(11);
  ep6in_txt->labelsize(11);
  ep6in_txt->readonly(1);

  ep6out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*4,40,18,"OUT-EP6");
  ep6out_txt->value("0");
  ep6out_txt->textsize(11);
  ep6out_txt->labelsize(11);
  ep6out_txt->align(FL_ALIGN_RIGHT);

  ep7in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*5,40,18,"IN-EP7");
  ep7in_txt->value("0");
  ep7in_txt->textsize(11);
  ep7in_txt->labelsize(11);
  ep7in_txt->readonly(1);

  ep7out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*5,40,18,"OUT-EP7");
  ep7out_txt->value("0");
  ep7out_txt->textsize(11);
  ep7out_txt->labelsize(11);
  ep7out_txt->align(FL_ALIGN_RIGHT);

  ep8in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*6,40,18,"IN-EP8");
  ep8in_txt->value("0");
  ep8in_txt->textsize(11);
  ep8in_txt->labelsize(11);
  ep8in_txt->readonly(1);

  ep8out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*6,40,18,"OUT-EP8");
  ep8out_txt->value("0");
  ep8out_txt->textsize(11);
  ep8out_txt->labelsize(11);
  ep8out_txt->align(FL_ALIGN_RIGHT);

  ep9in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*7,40,18,"IN-EP9");
  ep9in_txt->value("0");
  ep9in_txt->textsize(11);
  ep9in_txt->labelsize(11);
  ep9in_txt->readonly(1);

  ep9out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*7,40,18,"OUT-EP9");
  ep9out_txt->value("0");
  ep9out_txt->textsize(11);
  ep9out_txt->labelsize(11);
  ep9out_txt->align(FL_ALIGN_RIGHT);

  ep10in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*8,40,18,"IN-EP10");
  ep10in_txt->value("0");
  ep10in_txt->textsize(11);
  ep10in_txt->labelsize(11);
  ep10in_txt->readonly(1);

  ep10out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*8,40,18,"OUT-EP10");
  ep10out_txt->value("0");
  ep10out_txt->textsize(11);
  ep10out_txt->labelsize(11);
  ep10out_txt->align(FL_ALIGN_RIGHT);

  ep11in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*9,40,18,"IN-EP11");
  ep11in_txt->value("0");
  ep11in_txt->textsize(11);
  ep11in_txt->labelsize(11);
  ep11in_txt->readonly(1);

  ep11out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*9,40,18,"OUT-EP11");
  ep11out_txt->value("0");
  ep11out_txt->textsize(11);
  ep11out_txt->labelsize(11);
  ep11out_txt->align(FL_ALIGN_RIGHT);

  ep12in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*10,40,18,"IN-EP12");
  ep12in_txt->value("0");
  ep12in_txt->textsize(11);
  ep12in_txt->labelsize(11);
  ep12in_txt->readonly(1);

  ep12out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*10,40,18,"OUT-EP12");
  ep12out_txt->value("0");
  ep12out_txt->textsize(11);
  ep12out_txt->labelsize(11);
  ep12out_txt->align(FL_ALIGN_RIGHT);

  ep13in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*11,40,18,"IN-EP13");
  ep13in_txt->value("0");
  ep13in_txt->textsize(11);
  ep13in_txt->labelsize(11);
  ep13in_txt->readonly(1);

  ep13out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*11,40,18,"OUT-EP13");
  ep13out_txt->value("0");
  ep13out_txt->textsize(11);
  ep13out_txt->labelsize(11);
  ep13out_txt->align(FL_ALIGN_RIGHT);

  ep14in_txt=new Fl_Int_Input(this->x()+160,this->y()+55+20*12,40,18,"IN-EP14");
  ep14in_txt->value("0");
  ep14in_txt->textsize(11);
  ep14in_txt->labelsize(11);
  ep14in_txt->readonly(1);

  ep14out_txt=new Fl_Int_Input(this->x()+205,this->y()+55+20*12,40,18,"OUT-EP14");
  ep14out_txt->value("0");
  ep14out_txt->textsize(11);
  ep14out_txt->labelsize(11);
  ep14out_txt->align(FL_ALIGN_RIGHT);

  end();
  this->level_btn_cb();
  redraw();
}

void BIOEDeviceWidget::DeviceTest(void) {
   fl_message(titlebar_grp->label());
}

void BIOEDeviceWidget::setInterface(int port, int cs) {
  this->port=port;
  this->cs=cs;
  char label[30];
  sprintf(label, "Device%d, PORT=%d, CS=%d", this->adr,port,cs);
  titlebar_grp->copy_label(label);  
}

void BIOEDeviceWidget::processDevice() {
  long x;
  char buffer[20];
  int speedfactor=5;
  int pausefactor=30;
  BIOE_Init(port);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,2,atol(ep2out_txt->value()),speedfactor,pausefactor));
  ep2in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,3,atol(ep3out_txt->value()),speedfactor,pausefactor));
  ep3in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,4,atol(ep4out_txt->value()),speedfactor,pausefactor));
  ep4in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,5,atol(ep5out_txt->value()),speedfactor,pausefactor));
  ep5in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,6,atol(ep6out_txt->value()),speedfactor,pausefactor));
  ep6in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,7,atol(ep7out_txt->value()),speedfactor,pausefactor));
  ep7in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,8,atol(ep8out_txt->value()),speedfactor,pausefactor));
  ep8in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,9,atol(ep9out_txt->value()),speedfactor,pausefactor));
  ep9in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,10,atol(ep10out_txt->value()),speedfactor,pausefactor));
  ep10in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,11,atol(ep11out_txt->value()),speedfactor,pausefactor));
  ep11in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,12,atol(ep12out_txt->value()),speedfactor,pausefactor));
  ep12in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,13,atol(ep13out_txt->value()),speedfactor,pausefactor));
  ep13in_txt->value(buffer);
  sprintf(buffer,"%ld",(long)BIOE_Transaction(port,cs,this->adr,14,atol(ep14out_txt->value()),speedfactor,pausefactor));
  ep14in_txt->value(buffer);
  BIOE_Close(port);
}

int BIOEDeviceWidget::handle(int e) {
  Fl_Group* p;
  static int offset[2] = { 0, 0 };
  int ret = Fl_Group::handle(e);
  switch ( e ) {
    case FL_PUSH:
      offset[0] = x() - Fl::event_x();    // save where user clicked for dragging
      offset[1] = y() - Fl::event_y();
      p=this->parent();
      p->remove(this);
      p->add(this);
      return(1);
    case FL_RELEASE:
      this->damage(1,x()-1,y()-1,w()+2,h()+2);
      this->redraw();
      this->parent()->redraw();
      return(1);
    case FL_DRAG:
      position(offset[0]+Fl::event_x(), offset[1]+Fl::event_y());     // handle dragging
      this->damage(1,x()-1,y()-1,w()+2,h()+2);
      this->redraw();
      this->parent()->redraw();
      return(1);
  }
  return(ret);
}

void BIOEDeviceWidget::x_btn_cb() {
  this->parent()->redraw();
  this->parent()->remove(this);
}

void BIOEDeviceWidget::level_btn_cb() {
  DTN_txt->hide();
  setdtn_btn->hide();
  readsettings_btn->hide();
  s0_txt->hide();
  s1_txt->hide();
  s2_txt->hide();
  s3_txt->hide();
  s4_txt->hide();
  s5_txt->hide();
  s6_txt->hide();
  s7_txt->hide();
  writesettings_btn->hide();

  if (level_btn->value()==0) {
  }                           
  if (level_btn->value()==1) {
    DTN_txt->show();                             
    setdtn_btn->show();
  }                           
  if (level_btn->value()==2) {
    DTN_txt->show();
    setdtn_btn->show();
    readsettings_btn->show();
    s0_txt->show();
    s1_txt->show();
    s2_txt->show();
    s3_txt->show();
    s4_txt->show();
    s5_txt->show();
    s6_txt->show();
    s7_txt->show();
    writesettings_btn->show();                           
  }                           
}

void BIOEDeviceWidget::setdtn_btn_cb() {
  int DTN;
  int error=0;
  DTN=atoi(DTN_txt->value());
  BIOE_Init(this->port);
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0,DTN,2,20)<0) {
    error=1;
  }
  BIOE_Close(this->port);     
  if (error) {
   fl_message("Error writing DTN-Number!");             
  }
}

void BIOEDeviceWidget::readsettings_btn_cb() {
  char buffer[20];

  BIOE_Init(this->port);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x20,0,2,20));
  s0_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x21,0,2,20));
  s1_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x22,0,2,20));
  s2_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x23,0,2,20));
  s3_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x24,0,2,20));
  s4_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x25,0,2,20));
  s5_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x26,0,2,20));
  s6_txt->value(buffer);
  sprintf(buffer,"%ld",BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x27,0,2,20));
  s7_txt->value(buffer);
  BIOE_Close(this->port);     
}

void BIOEDeviceWidget::writesettings_btn_cb() {
  int error=0;
  BIOE_Init(this->port);
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x10,atoi(s0_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x11,atoi(s1_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x12,atoi(s2_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x13,atoi(s3_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x14,atoi(s4_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x15,atoi(s5_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x16,atoi(s6_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x17,atoi(s7_txt->value()),2,20)<0) error=1;
  if (BIOE_ConfigTransaction(this->port,this->cs,this->adr,0x02,1,2,20)<0) error=1;
  BIOE_Close(this->port);
  if (error) {
   fl_message("Error writing Settings!");             
  }  
}
