/*********************************************
 * BIOE-GUI - a frontend for the BIOE system, WIN32/UNIX
 * Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
 * 
 * License: CC-GNU GPL
 * http://creativecommons.org/licenses/GPL/2.0/ 
 *********************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *********************************************/

#ifndef _BIOEMain_H_
#define _BIOEMain_H_

#include <stdlib.h>
#include <stdio.h>
//#include <cstdio.h>

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Input.H>
#include <FL/fl_ask.H>
#include "BIOEDeviceWidget.h"
#include "xpmfiles.h"
#include "bioe.h"


class BIOEMain_Frm: public Fl_Double_Window {
  public:
    BIOEMain_Frm();
    ~BIOEMain_Frm();    
  private:
    Fl_Group      *menu_grp;
    Fl_Menu_Bar   *menu;

    
    Fl_Group      *toolbar_grp;
    Fl_Group      *grabber1_grp;
    Fl_Group      *grabber2_grp;
    Fl_Group      *spacer_grp;
    Fl_Button     *scandevices_btn;
    Fl_Button     *testdevice_btn;
    Fl_Button     *processalldevices_btn;
    Fl_Button     *processcurrentdevice_btn;

    Fl_Button     *help_btn;
    Fl_Button     *closeapp_btn;

    Fl_Group      *main_grp;
    Fl_Scroll     *workspace_grp;
    BIOEDeviceWidget *device0;
    BIOEDeviceWidget *device1;
    BIOEDeviceWidget *device2;
    BIOEDeviceWidget *device3;
    BIOEDeviceWidget *device4;
    BIOEDeviceWidget *device5;
    BIOEDeviceWidget *device6;
    BIOEDeviceWidget *device7;
    BIOEDeviceWidget *device8;
    BIOEDeviceWidget *device9;
    BIOEDeviceWidget *device10;
    BIOEDeviceWidget *device11;
    BIOEDeviceWidget *device12;
    BIOEDeviceWidget *device13;
    BIOEDeviceWidget *device14;
    BIOEDeviceWidget *device15;
    
    Fl_Group      *left_grp;
    Fl_Group      *interface_grp;
    Fl_Input      *lptport_txt;
    Fl_Input      *cs_txt;

    Fl_Scroll     *devices_grp;
    Fl_Button     *device0_btn;
    Fl_Button     *device1_btn;
    Fl_Button     *device2_btn;
    Fl_Button     *device3_btn;
    Fl_Button     *device4_btn;
    Fl_Button     *device5_btn;
    Fl_Button     *device6_btn;
    Fl_Button     *device7_btn;
    Fl_Button     *device8_btn;
    Fl_Button     *device9_btn;
    Fl_Button     *device10_btn;
    Fl_Button     *device11_btn;
    Fl_Button     *device12_btn;
    Fl_Button     *device13_btn;
    Fl_Button     *device14_btn;
    Fl_Button     *device15_btn;

    Fl_Group      *status_grp;

    static void processing_scb(void* data) {
	  ((BIOEMain_Frm*)data)->processing_cb();
    }
    void processing_cb(); 

    static void help_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->help_btn_cb();
    }
    void help_btn_cb();

    static void closeapp_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->closeapp_btn_cb();
    }
    void closeapp_btn_cb();

    static void scandevices_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->scandevices_btn_cb();
    }
    void scandevices_btn_cb();

    static void testdevice_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->testdevice_btn_cb();
    }
    void testdevice_btn_cb();

    static void processalldevices_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->processalldevices_btn_cb();
    }
    void processalldevices_btn_cb();

    static void processcurrentdevice_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->processcurrentdevice_btn_cb();
    }
    void processcurrentdevice_btn_cb();

    static void device0_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device0_btn_cb();
    }
    void device0_btn_cb();

    static void device1_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device1_btn_cb();
    }
    void device1_btn_cb();

    static void device2_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device2_btn_cb();
    }
    void device2_btn_cb();

    static void device3_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device3_btn_cb();
    }
    void device3_btn_cb();

    static void device4_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device4_btn_cb();
    }
    void device4_btn_cb();

    static void device5_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device5_btn_cb();
    }
    void device5_btn_cb();

    static void device6_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device6_btn_cb();
    }
    void device6_btn_cb();

    static void device7_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device7_btn_cb();
    }
    void device7_btn_cb();

    static void device8_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device8_btn_cb();
    }
    void device8_btn_cb();

    static void device9_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device9_btn_cb();
    }
    void device9_btn_cb();

    static void device10_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device10_btn_cb();
    }
    void device10_btn_cb();

    static void device11_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device11_btn_cb();
    }
    void device11_btn_cb();

    static void device12_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device12_btn_cb();
    }
    void device12_btn_cb();

    static void device13_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device13_btn_cb();
    }
    void device13_btn_cb();

    static void device14_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device14_btn_cb();
    }
    void device14_btn_cb();

    static void device15_btn_scb(Fl_Widget* w, void* data) {
	  ((BIOEMain_Frm*)data)->device15_btn_cb();
    }
    void device15_btn_cb();

};

#endif
