/*********************************************
 * BIOE-Library - communication library for the BIOE system, WIN32/UNIX
 * Copyright (C) 2010  Weichinger Klaus, snaky.1@gmx.at
 * 
 * License: CC-GNU GPL
 * http://creativecommons.org/licenses/GPL/2.0/ 
 *********************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef unix
  #include <sys/io.h>
#endif

#ifdef _WIN32
  #include <conio.h>
  #include <windows.h>
#endif

#include "bioe.h"

#define BIOELPT_CS3  0x01
#define BIOELPT_CS2  0x02
#define BIOELPT_CS1  0x04
#define BIOELPT_CLK  0x08
#define BIOELPT_DO0  0x10
#define BIOELPT_DO1  0x20
#define BIOELPT_DO2  0x40
#define BIOELPT_DO3  0x80
#define BIOELPT_DO   BIOELPT_DO0+BIOELPT_DO1+BIOELPT_DO2+BIOELPT_DO3

#define BIOELPT_DI0  0x40
#define BIOELPT_DI1  0x80
#define BIOELPT_DI2  0x20
#define BIOELPT_DI3  0x10
#define BIOELPT_DI   BIOELPT_DI0+BIOELPT_DI1+BIOELPT_DI2+BIOELPT_DI3

#define errorloops   3
#define errorloopsConfigTransaction    10000

#ifdef _WIN32
HINSTANCE hLib;

typedef short _stdcall (*inpfuncPtr)(short portaddr);
typedef void _stdcall (*oupfuncPtr)(short portaddr, short datum);

inpfuncPtr inp32;
oupfuncPtr oup32;
#endif

int open=0;

int BIOE_ReadWrite(int basePort, int Value, int CS, int CLK, int speedfactor) {
  int x=0,i,res=0;
  if (CS ==1) x|=BIOELPT_CS1;
  if (CS ==2) x|=BIOELPT_CS2;
  if (CS ==3) x|=BIOELPT_CS3;
  if ((Value&0x01)>0) x|=BIOELPT_DO0;
  if ((Value&0x02)>0) x|=BIOELPT_DO1;
  if ((Value&0x04)>0) x|=BIOELPT_DO2;
  if ((Value&0x08)>0) x|=BIOELPT_DO3;

  if (CLK!=0) {
    x|=BIOELPT_CLK;
    #ifdef unix
      outb(x,basePort);
    #endif
    #ifdef _WIN32
      (oup32)(basePort,x);  
    #endif
  }
  x&=~BIOELPT_CLK;
  #ifdef unix
    outb(x,basePort);
  #endif
  #ifdef _WIN32
    (oup32)(basePort,x);  
  #endif

  // Warte Schleife
  for (i=0;i<speedfactor;i++) {
    #ifdef unix
      x=inb(basePort+1);
    #endif
    
    #ifdef _WIN32
      x=(inp32)(basePort+1);    
    #endif
  }

  #ifdef unix
    x=inb(basePort+1);
  #endif
    
  #ifdef _WIN32
    x=(inp32)(basePort+1);    
  #endif
  res+=((x&BIOELPT_DI0)>0)?1:0;
  res+=((x&BIOELPT_DI1)>0)?0:2;
  res+=((x&BIOELPT_DI2)>0)?4:0;
  res+=((x&BIOELPT_DI3)>0)?8:0;
  return res;
}

int BIOE_Transaction(int basePort,int channel,int Device,int Endpoint,int Value, int speedfactor, int pausefactor) {
  int adr,ep,r3,r2,r1,r0,t3,t2,t1,t0,checksum,checksumread;
  int CS=channel;
  int valid,error=0;
  if (open==0) return -1;
  valid=0;
  while (valid==0) {
    BIOE_ReadWrite(basePort,0,CS,0,speedfactor);          // 1. Chip-Select setzen
    adr=Device&0x0F;
    ep=Endpoint&0x0F;
    BIOE_ReadWrite(basePort,adr,CS,1,speedfactor);     // 2. Ger�t ausw�hlen
    BIOE_ReadWrite(basePort,ep,CS,1,speedfactor);   // 3. Endpoint ausw�hlen
    t3=(int)(Value&0xF000)>>12;
    t2=(int)(Value&0x0F00)>>8;
    t1=(int)(Value&0x00F0)>>4;
    t0=(int)(Value&0x000F);
    r3=BIOE_ReadWrite(basePort,(int)(Value&0xF000)>>12,CS,1,speedfactor);          // 4. D3
    r2=BIOE_ReadWrite(basePort,(int)(Value&0x0F00)>>8,CS,1,speedfactor);           // 5. D2
    r1=BIOE_ReadWrite(basePort,(int)(Value&0x00F0)>>4,CS,1,speedfactor);           // 6. D1
    r0=BIOE_ReadWrite(basePort,(int)(Value&0x000F),CS,1,speedfactor);              // 7. D0
    checksum=adr^ep^r3^t3^r2^t2^r1^t1^r0^t0;
    checksumread=BIOE_ReadWrite(basePort,(int)(checksum&0x000F),CS,1,speedfactor);              // 7. D0
    if (checksum==checksumread) {
      valid=1;
    } else {
      error++;
      if (error==errorloops) {
        valid=1;
      }
    }
    BIOE_ReadWrite(basePort,0,0,0,pausefactor);           // 8. Chip-Select l�schenhen
  }
  if (error==errorloops) {
    return -1;
  } else {
    return r0+r1*16+r2*256+r3*4096;
  }
}

int BIOE_ConfigTransaction(int basePort, int channel, int Device, int Action, int Data, int speedfactor, int pausefactor) {
    int errorcount;
    int cmd,read;

    if (open==0) return -1;

    // Schreibt Command-Reset
    BIOE_Transaction(basePort, channel, Device, 0, 0xFFFF,speedfactor,pausefactor);
    errorcount=errorloopsConfigTransaction;
    while (BIOE_Transaction(basePort, channel, Device, 1, 0xFFFF,speedfactor,pausefactor)!=0xFFFF) {
      errorcount--;
      if (errorcount==0) {
        return -1;
      }
    }

    // Schreibt neuen Befehl
    cmd=Action*256+(Data&0x00FF);
    BIOE_Transaction(basePort, channel, Device, 0,cmd,speedfactor,pausefactor);
    errorcount=errorloopsConfigTransaction;
    read=BIOE_Transaction(basePort, channel, Device, 1, 0xFFFF,speedfactor,pausefactor);
    while ((read&0xFF00)!=(cmd&0xFF00)) {
      read=BIOE_Transaction(basePort, channel, Device, 1, 0xFFFF,speedfactor,pausefactor);
      errorcount--;
      if (errorcount==0) {
        return -1;
      }
    }
    return (read&0x00FF);
}

void BIOE_Init(int basePort) {
  int res;
  if (open!=0) return; // still open
  #ifdef unix
    ioperm(basePort,3,0);
    ioperm(basePort,3,1);
    res=0;
  #endif
  #ifdef _WIN32
  
    hLib = LoadLibrary("inpout32.dll");
    res=0;
    if (hLib == NULL) {
      res=-1;
    }
    inp32 = (inpfuncPtr) GetProcAddress(hLib, "Inp32");

    if (inp32 == NULL) {
      res=-1;
    }

    oup32 = (oupfuncPtr) GetProcAddress(hLib, "Out32");

    if (oup32 == NULL) {
      res=-1;
    }
    
  #endif
  if (res==0) open=1;
  BIOE_ReadWrite(basePort,0,0,0,0);           // 8. Chip-Select l�schen       
}

void BIOE_Close(int basePort) {
  if (open==0) return; // still closed
     
  #ifdef unix
    ioperm(basePort,3,0);
  #endif
  #ifdef _WIN32
    FreeLibrary(hLib);           
  #endif
  open=0;
}

int BIOE_DeviceExists(int basePort, int channel, int Device) {
  int read1,read2;
  BIOE_Transaction(basePort,channel,Device,15,0x1313,10,50);
  read1=BIOE_Transaction(basePort,channel,Device,15,0xAFAF,10,50);
  read2=BIOE_Transaction(basePort,channel,Device,15,0x0000,10,50);
  if ((read1==0x1313) && (read2==0xAFAF)) return 0;
  return -1;
}

void BIOE_WriteConfigFile(char *filename, char *ID, int port, int cs, int speedfactor, int pausefactor) {
  FILE *fp;
  fp=fopen(filename,"w");
  if (fp==NULL) {
  } else {
    fprintf(fp,"bioe_port%s=%d\n",ID,port);
    fprintf(fp,"bioe_cs%s=%d\n",ID,cs);
    fprintf(fp,"bioe_speedfactor%s=%d\n",ID,speedfactor);
    fprintf(fp,"bioe_pausefactor%s=%d\n",ID,pausefactor);
    fclose(fp);
  }
}

int BIOE_ReadConfigFile_Int(char *filename, char* paramname, int defaultvalue) {
  char line[256];
  char val[256];
  int value;
  int state;
  int i,j;
  FILE *fp;
  
  fp = fopen(filename,"r");
  value=defaultvalue;
  if(fp == NULL) {
	  // configuration file don't exists
  } else {
    while (fgets(line,sizeof(line)-1,fp) != NULL) {
      if (line[0]=='#') {
      } else {
        i=0;
        j=0;
        state=1; // 1 ... parsing name
                 // 2 ... parsing space until "=" found
                 // 3 ... parsing "=" until something else found
                 // 4 ... parsing space until something else found
                 // 5 ... parsing value
                 // always break if char 10 or char 13 found
        val[j]=0;
        while (1) {
          if ((line[i]==0)||(line[i]==13)||(line[i]==10)) break;
          if (state==1) {
            if (paramname[i]==0) {
              if ((line[i]=='=')||(line[i]==' ')) {
                state=2;
              } else {
                break;
              }
            } else {
              if (line[i]!=paramname[i]) break;
            }
          }
          if (state==2) {
            if (line[i]!=' ') {
              state=3;
            }
          }
          if (state==3) {
            if (line[i]!='=') {
              state=4;
            }
          }
          if (state==4) {
            if (line[i]!=' ') {
              state=5;
            }
          }
          if (state==5) {
            val[j]=line[i];
            j++;
            val[j]=0;
          }
          i++;          
        }
        if (state==5) {
          value=atoi(val);
        }
      }
    }
    fclose(fp);
  }
  return value;
}
